import express, { Request, Response } from "express";
import { createServer, Server as HTTPServer } from "http";
import { Server as SocketIOServer, Socket } from "socket.io";
import cors from "cors";

// Initialize Express
const app = express();

// Set up CORS middleware for Express
app.use(
  cors({
    origin: "http://localhost:4200", // Update this to the origin of your client
    methods: ["GET", "POST"],
  })
);

// Create an HTTP server
const httpServer: HTTPServer = createServer(app);

// Initialize Socket.io on the server
const io = new SocketIOServer(httpServer, {
  path: "/socket", // Define the WebSocket path
  cors: {
    origin: "http://localhost:4200", // Update this to the origin of your client
    methods: ["GET", "POST"],
  },
});

// Store online users
const onlineUsers: Map<string, any> = new Map();

// Handle WebSocket connections
io.on("connection", (socket: Socket) => {
  console.log("User connected", socket.id);

  // Handle user-online event
  socket.on("user-online", (user: any) => {
    // Now we store the socket ID along with the user details
    onlineUsers.set(user, { userId: user, socketId: socket.id });
    // io.emit("online-users", Array.from(onlineUsers.values()));
  });

  // Handle user disconnect event
  socket.on("disconnect", () => {
    const userArr = Array.from(onlineUsers.values());
    const u = userArr.filter((item: any) => item.socketId == socket.id);
    onlineUsers.delete(u[0].userId);
  });

  socket.on("ice-candidate", (data) => {
    const { candidate, to } = data;
    const calledTo = onlineUsers.get(to);
    if (calledTo) {
      // Emit to the recipient's socket ID
      socket.to(calledTo.socketId).emit("get-ice-candidate", candidate);
    }
  });

  // // Handle call-related events
  socket.on("offer", (data) => {
    const { offer, to, sender } = data;
    const calledTo = onlineUsers.get(to);
    if (calledTo) {
      // Emit to the recipient's socket ID
      socket.to(calledTo.socketId).emit("get-offer", { offer, sender: sender });
    }
  });

  socket.on("call-request", (data) => {
    const { answer, to } = data;
    const calledTo = onlineUsers.get(to);
    if (calledTo) {
      socket.to(calledTo.socketId).emit("get-answer", answer);
    }
  });

  socket.on("call-accept", (data) => {
    const { to } = data;
    const calledTo = onlineUsers.get(to);
    if (calledTo) {
      socket.to(calledTo.socketId).emit("call-accepted");
    }
  });

  socket.on("call-reject", (data: { to: string }) => {
    const { to } = data;
    const calledTo = onlineUsers.get(to);
    if (calledTo) {
      socket.to(calledTo.socketId).emit("call-rejected");
    }
  });
});

// Define a basic route
app.get("/", (req: Request, res: Response) => {
  res.send("WebSocket server is running");
});

// Start the server on port 3000
const PORT = process.env.PORT || 8000;
httpServer.listen(PORT, () => {
  console.log(`Server is running on port ${PORT}`);
});
